@@ -1,7 +1,6 @@
 /* Distributed under the OSI-approved BSD 3-Clause License.  See accompanying
    file Copyright.txt or https://cmake.org/licensing for details.  */
-#ifndef cmTryRunCommand_h
-#define cmTryRunCommand_h
+#pragma once
 
 #include "cmConfigure.h" // IWYU pragma: keep
 
@@ -50,6 +49,5 @@ private:
   std::string OutputVariable;
   std::string RunOutputVariable;
   std::string CompileOutputVariable;
+  std::string WorkingDirectory;
 };
-
-#endif
